@@ -1,5 +1,5 @@
 #![allow(dead_code)]
 
-pub const BRANCH: &str = "unstable";
+pub const BRANCH: &str = "v0.1";
 pub const USER: &str = "Tamschi";
 pub const RUST_VERSION: &str = "1.46";
